@@ -24,147 +24,125 @@ description:
 options:
   command:
     description:
-      - Specifies the action to take.  
+      - Specifies the action to take. The 'reboot' option is available starting at version 2.0
     required: true
-    default: null
-    aliases: []
-    choices: [ 'create', 'replicate', 'delete', 'facts', 'modify' , 'promote', 'snapshot', 'restore' ]
+    choices: [ 'create', 'replicate', 'delete', 'facts', 'modify' , 'promote', 'snapshot', 'reboot', 'restore' ]
   instance_name:
     description:
       - Database instance identifier. Required except when using command=facts or command=delete on just a snapshot
     required: false
     default: null
-    aliases: []
   source_instance:
     description:
       - Name of the database to replicate. Used only when command=replicate.
     required: false
     default: null
-    aliases: []
   db_engine:
     description:
-      - The type of database.  Used only when command=create. 
+      - The type of database.  Used only when command=create.
     required: false
     default: null
-    aliases: []
     choices: [ 'MySQL', 'oracle-se1', 'oracle-se', 'oracle-ee', 'sqlserver-ee', 'sqlserver-se', 'sqlserver-ex', 'sqlserver-web', 'postgres']
   size:
     description:
       - Size in gigabytes of the initial storage for the DB instance. Used only when command=create or command=modify.
     required: false
     default: null
-    aliases: []
   instance_type:
     description:
-      - The instance type of the database.  Must be specified when command=create. Optional when command=replicate, command=modify or command=restore. If not specified then the replica inherits the same instance type as the source instance. 
+      - The instance type of the database.  Must be specified when command=create. Optional when command=replicate, command=modify or command=restore. If not specified then the replica inherits the same instance type as the source instance.
     required: false
     default: null
-    aliases: []
   username:
     description:
       - Master database username. Used only when command=create.
     required: false
     default: null
-    aliases: []
   password:
     description:
       - Password for the master database username. Used only when command=create or command=modify.
     required: false
     default: null
-    aliases: []
   region:
     description:
       - The AWS region to use. If not specified then the value of the EC2_REGION environment variable, if any, is used.
     required: true
-    default: null
     aliases: [ 'aws_region', 'ec2_region' ]
   db_name:
     description:
       - Name of a database to create within the instance.  If not specified then no database is created. Used only when command=create.
     required: false
     default: null
-    aliases: []
   engine_version:
     description:
       - Version number of the database engine to use. Used only when command=create. If not specified then the current Amazon RDS default engine version is used.
     required: false
     default: null
-    aliases: []
   parameter_group:
     description:
       - Name of the DB parameter group to associate with this instance.  If omitted then the RDS default DBParameterGroup will be used. Used only when command=create or command=modify.
     required: false
     default: null
-    aliases: []
   license_model:
     description:
-      - The license model for this DB instance. Used only when command=create or command=restore. 
+      - The license model for this DB instance. Used only when command=create or command=restore.
     required: false
     default: null
-    aliases: []
-    choices:  [ 'license-included', 'bring-your-own-license', 'general-public-license' ]
+    choices:  [ 'license-included', 'bring-your-own-license', 'general-public-license', 'postgresql-license' ]
   multi_zone:
     description:
       - Specifies if this is a Multi-availability-zone deployment. Can not be used in conjunction with zone parameter. Used only when command=create or command=modify.
-    choices: [ "yes", "no" ] 
+    choices: [ "yes", "no" ]
     required: false
     default: null
-    aliases: []
   iops:
     description:
       - Specifies the number of IOPS for the instance.  Used only when command=create or command=modify. Must be an integer greater than 1000.
     required: false
     default: null
-    aliases: []
   security_groups:
     description:
       - Comma separated list of one or more security groups.  Used only when command=create or command=modify.
     required: false
     default: null
-    aliases: []
   vpc_security_groups:
     description:
       - Comma separated list of one or more vpc security group ids. Also requires `subnet` to be specified. Used only when command=create or command=modify.
     required: false
     default: null
-    aliases: []
   port:
     description:
-      - Port number that the DB instance uses for connections.  Defaults to 3306 for mysql. Must be changed to 1521 for Oracle, 1443 for SQL Server, 5432 for PostgreSQL. Used only when command=create or command=replicate.
+      - Port number that the DB instance uses for connections. Used only when command=create or command=replicate.
+      - Prior to 2.0 it always defaults to null and the API would use 3306, it had to be set to other DB default values when not using MySql.
+        Starting at 2.0 it auotmaticaly defaults to what is expected for each c(db_engine).
     required: false
-    default: null
-    aliases: []
+    default: 3306 for mysql, 1521 for Oracle, 1433 for SQL Server, 5432 for PostgreSQL.
   upgrade:
     description:
-      - Indicates that minor version upgrades should be applied automatically. Used only when command=create or command=replicate. 
+      - Indicates that minor version upgrades should be applied automatically. Used only when command=create or command=replicate.
     required: false
     default: no
     choices: [ "yes", "no" ]
-    aliases: []
   option_group:
     description:
       - The name of the option group to use.  If not specified then the default option group is used. Used only when command=create.
     required: false
     default: null
-    aliases: []
   maint_window:
     description:
       - "Maintenance window in format of ddd:hh24:mi-ddd:hh24:mi.  (Example: Mon:22:00-Mon:23:15) If not specified then a random maintenance window is assigned. Used only when command=create or command=modify."
     required: false
     default: null
-    aliases: []
   backup_window:
     description:
       - Backup window in format of hh24:mi-hh24:mi.  If not specified then a random backup window is assigned. Used only when command=create or command=modify.
     required: false
     default: null
-    aliases: []
   backup_retention:
     description:
       - "Number of days backups are retained.  Set to 0 to disable backups.  Default is 1 day.  Valid range: 0-35. Used only when command=create or command=modify."
     required: false
     default: null
-    aliases: []
   zone:
     description:
       - availability zone in which to launch the instance. Used only when command=create, command=replicate or command=restore.
@@ -176,18 +154,15 @@ options:
       - VPC subnet group.  If specified then a VPC instance is created. Used only when command=create.
     required: false
     default: null
-    aliases: []
   snapshot:
     description:
       - Name of snapshot to take. When command=delete, if no snapshot name is provided then no snapshot is taken. If used with command=delete with no instance_name, the snapshot is deleted. Used with command=facts, command=delete or command=snapshot.
     required: false
     default: null
-    aliases: []
   aws_secret_key:
     description:
       - AWS secret key. If not set then the value of the AWS_SECRET_KEY environment variable is used. 
     required: false
-    default: null
     aliases: [ 'ec2_secret_key', 'secret_key' ]
   aws_access_key:
     description:
@@ -201,48 +176,53 @@ options:
     required: false
     default: "no"
     choices: [ "yes", "no" ]
-    aliases: []
   wait_timeout:
     description:
       - how long before wait gives up, in seconds
     default: 300
-    aliases: []
   apply_immediately:
     description:
       - Used only when command=modify.  If enabled, the modifications will be applied as soon as possible rather than waiting for the next preferred maintenance window.
     default: no
     choices: [ "yes", "no" ]
-    aliases: []
+  force_failover:
+    description:
+      - Used only when command=reboot.  If enabled, the reboot is done using a MultiAZ failover.
+    required: false
+    default: "no"
+    choices: [ "yes", "no" ]
+    version_added: "2.0"
   new_instance_name:
     description:
       - Name to rename an instance to. Used only when command=modify.
     required: false
     default: null
-    aliases: []
-    version_added: 1.5
+    version_added: "1.5"
   character_set_name:
     description:
       - Associate the DB instance with a specified character set. Used with command=create.
     required: false
     default: null
-    aliases: []
-    version_added: 1.9
+    version_added: "1.9"
   publicly_accessible:
     description:
       - explicitly set whether the resource should be publicly accessible or not. Used with command=create, command=replicate. Requires boto >= 2.26.0
     required: false
     default: null
-    aliases: []
-    version_added: 1.9
+    version_added: "1.9"
   tags:
     description:
       - tags dict to apply to a resource. Used with command=create, command=replicate, command=restore. Requires boto >= 2.26.0
     required: false
     default: null
-    aliases: []
-    version_added: 1.9
-requirements: [ "boto" ]
-author: Bruce Pennypacker, Will Thames
+    version_added: "1.9"
+requirements:
+    - "python >= 2.6"
+    - "boto"
+author:
+    - "Bruce Pennypacker (@bpennypacker)"
+    - "Will Thames (@willthames)"
+
 '''
 
 # FIXME: the command stuff needs a 'state' like alias to make things consistent -- MPD
@@ -287,6 +267,40 @@ EXAMPLES = '''
     instance_name: new-database
     new_instance_name: renamed-database
     wait: yes
+
+# Reboot an instance and wait for it to become available again
+- rds
+    command: reboot
+    instance_name: database
+    wait: yes
+    
+# Restore a Postgres db instance from a snapshot, wait for it to become available again, and
+#  then modify it to add your security group. Also, display the new endpoint.
+#  Note that the "publicly_accessible" option is allowed here just as it is in the AWS CLI
+- local_action:
+     module: rds
+     command: restore
+     snapshot: mypostgres-snapshot
+     instance_name: MyNewInstanceName
+     region: us-west-2
+     zone: us-west-2b
+     subnet: default-vpc-xx441xxx
+     publicly_accessible: yes
+     wait: yes
+     wait_timeout: 600
+     tags:
+         Name: pg1_test_name_tag 
+  register: rds
+  
+- local_action:
+     module: rds
+     command: modify
+     instance_name: MyNewInstanceName
+     region: us-west-2
+     vpc_security_groups: sg-xxx945xx
+          
+- debug: msg="The new db endpoint is {{ rds.instance.endpoint }}"
+
 '''
 
 import sys
@@ -294,9 +308,9 @@ import time
 
 try:
     import boto.rds
+    HAS_BOTO = True
 except ImportError:
-    print "failed=True msg='boto required for this module'"
-    sys.exit(1)
+    HAS_BOTO = False
 
 try:
     import boto.rds2
@@ -304,6 +318,12 @@ try:
 except ImportError:
     has_rds2 = False
 
+DEFAULT_PORTS= {
+    'mysql': 3306,
+    'oracle': 1521,
+    'sqlserver': 1433,
+    'postgres': 5432,
+}
 
 class RDSException(Exception):
     def __init__(self, exc):
@@ -332,7 +352,7 @@ class RDSConnection:
             return None
 
     def get_db_snapshot(self, snapshotid):
-        try: 
+        try:
             return RDSSnapshot(self.connection.get_all_dbsnapshots(snapshot_id=snapshotid)[0])
         except boto.exception.BotoServerError, e:
             return None
@@ -375,6 +395,13 @@ class RDSConnection:
         except boto.exception.BotoServerError, e:
             raise RDSException(e)
 
+    def reboot_db_instance(self, instance_name, **params):
+        try:
+            result = self.connection.reboot_dbinstance(instance_name)
+            return RDSDBInstance(result)
+        except boto.exception.BotoServerError, e:
+            raise RDSException(e)
+
     def restore_db_instance_from_db_snapshot(self, instance_name, snapshot, instance_type, **params):
         try:
             result = self.connection.restore_dbinstance_from_dbsnapshot(snapshot, instance_name, instance_type, **params)
@@ -459,6 +486,13 @@ class RDS2Connection:
         except boto.exception.BotoServerError, e:
             raise RDSException(e)
 
+    def reboot_db_instance(self, instance_name, **params):
+        try:
+            result = self.connection.reboot_db_instance(instance_name, **params)['RebootDBInstanceResponse']['RebootDBInstanceResult']['DBInstance']
+            return RDS2DBInstance(result)
+        except boto.exception.BotoServerError, e:
+            raise RDSException(e)
+
     def restore_db_instance_from_db_snapshot(self, instance_name, snapshot, instance_type, **params):
         try:
             result = self.connection.restore_db_instance_from_db_snapshot(instance_name, snapshot, **params)['RestoreDBInstanceFromDBSnapshotResponse']['RestoreDBInstanceFromDBSnapshotResult']['DBInstance']
@@ -611,17 +645,19 @@ def await_resource(conn, resource, status, module):
     while wait_timeout > time.time() and resource.status != status:
         time.sleep(5)
         if wait_timeout <= time.time():
-            module.fail_json(msg="Timeout waiting for resource %s" % resource.id)
+            module.fail_json(msg="Timeout waiting for RDS resource %s" % resource.name)
         if module.params.get('command') == 'snapshot':
             # Temporary until all the rds2 commands have their responses parsed
             if resource.name is None:
-                module.fail_json(msg="Problem with snapshot %s" % resource.snapshot)
+                module.fail_json(msg="There was a problem waiting for RDS snapshot %s" % resource.snapshot)
             resource = conn.get_db_snapshot(resource.name)
         else:
             # Temporary until all the rds2 commands have their responses parsed
             if resource.name is None:
-                module.fail_json(msg="Problem with instance %s" % resource.instance)
+                module.fail_json(msg="There was a problem waiting for RDS instance %s" % resource.instance)
             resource = conn.get_db_instance(resource.name)
+            if resource is None:
+                break
     return resource
 
 
@@ -652,7 +688,7 @@ def create_db_instance(module, conn):
                     module.params.get('username'), module.params.get('password'), **params)
             changed = True
         except RDSException, e:
-            module.fail_json(msg="failed to create instance: %s" % e.message)
+            module.fail_json(msg="Failed to create instance: %s" % e.message)
 
     if module.params.get('wait'):
         resource = await_resource(conn, result, 'available', module)
@@ -679,7 +715,7 @@ def replicate_db_instance(module, conn):
             result = conn.create_db_instance_read_replica(instance_name, source_instance, **params)
             changed = True
         except RDSException, e:
-            module.fail_json(msg="failed to create replica instance: %s " % e.message)
+            module.fail_json(msg="Failed to create replica instance: %s " % e.message)
 
     if module.params.get('wait'):
         resource = await_resource(conn, result, 'available', module)
@@ -708,14 +744,17 @@ def delete_db_instance_or_snapshot(module, conn):
         if instance_name:
             if snapshot:
                 params["skip_final_snapshot"] = False
-                params["final_snapshot_id"] = snapshot
+                if has_rds2:
+                    params["final_db_snapshot_identifier"] = snapshot
+                else:
+                    params["final_snapshot_id"] = snapshot
             else:
                 params["skip_final_snapshot"] = True
             result = conn.delete_db_instance(instance_name, **params)
         else:
             result = conn.delete_db_snapshot(snapshot)
     except RDSException, e:
-        module.fail_json(msg="failed to delete instance: %s" % e.message)
+        module.fail_json(msg="Failed to delete instance: %s" % e.message)
 
     # If we're not waiting for a delete to complete then we're all done
     # so just return
@@ -741,11 +780,11 @@ def facts_db_instance_or_snapshot(module, conn):
     snapshot = module.params.get('snapshot')
 
     if instance_name and snapshot:
-        module.fail_json(msg="facts must be called with either instance_name or snapshot, not both")
+        module.fail_json(msg="Facts must be called with either instance_name or snapshot, not both")
     if instance_name:
         resource = conn.get_db_instance(instance_name)
         if not resource:
-            module.fail_json(msg="DB Instance %s does not exist" % instance_name)
+            module.fail_json(msg="DB instance %s does not exist" % instance_name)
     if snapshot:
         resource = conn.get_db_snapshot(snapshot)
         if not resource:
@@ -759,7 +798,7 @@ def modify_db_instance(module, conn):
     valid_vars = ['apply_immediately', 'backup_retention', 'backup_window',
                   'db_name', 'engine_version', 'instance_type', 'iops', 'license_model',
                   'maint_window', 'multi_zone', 'new_instance_name',
-                  'option_group', 'parameter_group' 'password', 'size', 'upgrade']
+                  'option_group', 'parameter_group', 'password', 'size', 'upgrade']
 
     params = validate_parameters(required_vars, valid_vars, module)
     instance_name = module.params.get('instance_name')
@@ -798,13 +837,17 @@ def promote_db_instance(module, conn):
     instance_name = module.params.get('instance_name')
     
     result = conn.get_db_instance(instance_name)
+    if not result:
+        module.fail_json(msg="DB Instance %s does not exist" % instance_name)
+
     if result.get_data().get('replication_source'):
-        changed = False
-    else:
         try:
             result = conn.promote_read_replica(instance_name, **params)
+            changed = True
         except RDSException, e:
             module.fail_json(msg=e.message)
+    else:
+        changed = False
 
     if module.params.get('wait'):
         resource = await_resource(conn, result, 'available', module)
@@ -837,6 +880,31 @@ def snapshot_db_instance(module, conn):
     module.exit_json(changed=changed, snapshot=resource.get_data())
 
 
+def reboot_db_instance(module, conn):
+    required_vars = ['instance_name']
+    valid_vars = []
+
+    if has_rds2:
+        valid_vars.append('force_failover')
+
+    params = validate_parameters(required_vars, valid_vars, module)
+    instance_name = module.params.get('instance_name')
+    result = conn.get_db_instance(instance_name)
+    changed = False
+    try:
+        result = conn.reboot_db_instance(instance_name, **params)
+        changed = True
+    except RDSException, e:
+        module.fail_json(msg=e.message)
+
+    if module.params.get('wait'):
+        resource = await_resource(conn, result, 'available', module)
+    else:
+        resource = conn.get_db_instance(instance_name)
+
+    module.exit_json(changed=changed, instance=resource.get_data())
+
+
 def restore_db_instance(module, conn):
     required_vars = ['instance_name', 'snapshot']
     valid_vars = ['db_name', 'iops', 'license_model', 'multi_zone',
@@ -888,6 +956,7 @@ def validate_parameters(required_vars, valid_vars, module):
             'subnet': 'db_subnet_group_name',
             'license_model': 'license_model',
             'option_group': 'option_group_name',
+            'size': 'allocated_storage',
             'iops': 'iops',
             'new_instance_name': 'new_instance_id',
             'apply_immediately': 'apply_immediately',
@@ -908,6 +977,7 @@ def validate_parameters(required_vars, valid_vars, module):
             'instance_type': 'db_instance_class',
             'password': 'master_user_password',
             'new_instance_name': 'new_db_instance_identifier',
+            'force_failover': 'force_failover',
     }
     if has_rds2:
         optional_params.update(optional_params_rds2)
@@ -931,11 +1001,15 @@ def validate_parameters(required_vars, valid_vars, module):
     if module.params.get('security_groups'):
         params[sec_group] = module.params.get('security_groups').split(',')
 
-    if module.params.get('vpc_security_groups'):
-        groups_list = []
-        for x in module.params.get('vpc_security_groups'):
-            groups_list.append(boto.rds.VPCSecurityGroupMembership(vpc_group=x))
-        params["vpc_security_groups"] = groups_list
+    vpc_groups = module.params.get('vpc_security_groups')
+    if vpc_groups:
+        if has_rds2:
+            params['vpc_security_group_ids'] = vpc_groups
+        else:
+            groups_list = []
+            for x in vpc_groups:
+                groups_list.append(boto.rds.VPCSecurityGroupMembership(vpc_group=x))
+            params['vpc_security_groups'] = groups_list
 
     # Convert tags dict to list of tuples that rds2 expects
     if 'tags' in params:
@@ -946,7 +1020,7 @@ def validate_parameters(required_vars, valid_vars, module):
 def main():
     argument_spec = ec2_argument_spec()
     argument_spec.update(dict(
-            command           = dict(choices=['create', 'replicate', 'delete', 'facts', 'modify', 'promote', 'snapshot', 'restore'], required=True),
+            command           = dict(choices=['create', 'replicate', 'delete', 'facts', 'modify', 'promote', 'snapshot', 'reboot', 'restore'], required=True),
             instance_name     = dict(required=False),
             source_instance   = dict(required=False),
             db_engine         = dict(choices=['MySQL', 'oracle-se1', 'oracle-se', 'oracle-ee', 'sqlserver-ee', 'sqlserver-se', 'sqlserver-ex', 'sqlserver-web', 'postgres'], required=False),
@@ -957,9 +1031,9 @@ def main():
             db_name           = dict(required=False),
             engine_version    = dict(required=False),
             parameter_group   = dict(required=False),
-            license_model     = dict(choices=['license-included', 'bring-your-own-license', 'general-public-license'], required=False),
+            license_model     = dict(choices=['license-included', 'bring-your-own-license', 'general-public-license', 'postgresql-license'], required=False),
             multi_zone        = dict(type='bool', default=False),
-            iops              = dict(required=False), 
+            iops              = dict(required=False),
             security_groups   = dict(required=False),
             vpc_security_groups = dict(type='list', required=False),
             port              = dict(required=False),
@@ -967,7 +1041,7 @@ def main():
             option_group      = dict(required=False),
             maint_window      = dict(required=False),
             backup_window     = dict(required=False),
-            backup_retention  = dict(required=False), 
+            backup_retention  = dict(required=False),
             zone              = dict(aliases=['aws_zone', 'ec2_zone'], required=False),
             subnet            = dict(required=False),
             wait              = dict(type='bool', default=False),
@@ -978,12 +1052,17 @@ def main():
             tags              = dict(type='dict', required=False),
             publicly_accessible = dict(required=False),
             character_set_name = dict(required=False),
+            force_failover    = dict(type='bool', required=False, default=False)
         )
     )
 
     module = AnsibleModule(
         argument_spec=argument_spec,
     )
+
+    if not HAS_BOTO:
+        module.fail_json(msg='boto required for this module')
+
     invocations = {
             'create': create_db_instance,
             'replicate': replicate_db_instance,
@@ -992,12 +1071,21 @@ def main():
             'modify': modify_db_instance,
             'promote': promote_db_instance,
             'snapshot': snapshot_db_instance,
+            'reboot': reboot_db_instance,
             'restore': restore_db_instance,
     }
 
     region, ec2_url, aws_connect_params = get_aws_connection_info(module)
     if not region:
-        module.fail_json(msg="region not specified and unable to determine region from EC2_REGION.")
+        module.fail_json(msg="Region not specified. Unable to determine region from EC2_REGION.")
+
+    # set port to per db defaults if not specified
+    if module.params['port'] is None and module.params['db_engine'] is not None and module.params['command'] == 'create':
+        if '-' in module.params['db_engine']:
+            engine = module.params['db_engine'].split('-')[0]
+        else:
+            engine = module.params['db_engine']
+        module.params['port'] = DEFAULT_PORTS[engine.lower()]
 
     # connect to the rds endpoint
     if has_rds2:
@@ -1006,7 +1094,7 @@ def main():
         conn = RDSConnection(module, region, **aws_connect_params)
 
     invocations[module.params.get('command')](module, conn)
-        
+
 # import module snippets
 from ansible.module_utils.basic import *
 from ansible.module_utils.ec2 import *
