@@ -22,7 +22,7 @@ module: gce_net
 version_added: "1.5"
 short_description: create/destroy GCE networks and firewall rules
 description:
-    - This module can create and destroy Google Compue Engine networks and
+    - This module can create and destroy Google Compute Engine networks and
       firewall rules U(https://developers.google.com/compute/docs/networking).
       The I(name) parameter is reserved for referencing a network while the
       I(fwname) parameter is used to reference firewall rules.
@@ -33,13 +33,14 @@ description:
 options:
   allowed:
     description:
-      - the protocol:ports to allow ('tcp:80' or 'tcp:80,443' or 'tcp:80-800')
+      - the protocol:ports to allow ('tcp:80' or 'tcp:80,443' or 'tcp:80-800;udp:1-25')
     required: false
     default: null
     aliases: []
   ipv4_range:
     description:
       - the IPv4 address range in CIDR notation for the network
+        this parameter is not mandatory when you specified existing network in name parameter, but when you create new network, this parameter is mandatory
     required: false
     aliases: ['cidr']
   fwname:
@@ -75,7 +76,7 @@ options:
     aliases: []
   state:
     description:
-      - desired state of the persistent disk
+      - desired state of the network or firewall
     required: false
     default: "present"
     choices: ["active", "present", "absent", "deleted"]
@@ -91,6 +92,14 @@ options:
     version_added: "1.6"
     description:
       - path to the pem file associated with the service account email
+        This option is deprecated. Use 'credentials_file'.
+    required: false
+    default: null
+    aliases: []
+  credentials_file:
+    version_added: "2.1.0"
+    description:
+      - path to the JSON file associated with the service account email
     required: false
     default: null
     aliases: []
@@ -102,8 +111,10 @@ options:
     default: null
     aliases: []
 
-requirements: [ "libcloud" ]
-author: Eric Johnson <erjohnso@google.com>
+requirements:
+    - "python >= 2.6"
+    - "apache-libcloud >= 0.13.3, >= 0.17.0 if using JSON credentials"
+author: "Eric Johnson (@erjohnso) <erjohnso@google.com>"
 '''
 
 EXAMPLES = '''
@@ -123,22 +134,18 @@ EXAMPLES = '''
 
 '''
 
-import sys
-
 try:
     from libcloud.compute.types import Provider
     from libcloud.compute.providers import get_driver
     from libcloud.common.google import GoogleBaseError, QuotaExceededError, \
             ResourceExistsError, ResourceNotFoundError
     _ = Provider.GCE
+    HAS_LIBCLOUD = True
 except ImportError:
-    print("failed=True " + \
-            "msg='libcloud with GCE support required for this module.'")
-    sys.exit(1)
+    HAS_LIBCLOUD = False
 
-
-def format_allowed(allowed):
-    """Format the 'allowed' value so that it is GCE compatible."""
+def format_allowed_section(allowed):
+    """Format each section of the allowed list"""
     if allowed.count(":") == 0:
         protocol = allowed
         ports = []
@@ -153,8 +160,18 @@ def format_allowed(allowed):
     return_val = {"IPProtocol": protocol}
     if ports:
         return_val["ports"] = ports
-    return [return_val]
+    return return_val
 
+def format_allowed(allowed):
+    """Format the 'allowed' value so that it is GCE compatible."""
+    return_value = []
+    if allowed.count(";") == 0:
+        return [format_allowed_section(allowed)]
+    else:
+        sections = allowed.split(";")
+        for section in sections:
+            return_value.append(format_allowed_section(section))
+    return return_value
 
 def main():
     module = AnsibleModule(
@@ -169,10 +186,14 @@ def main():
             state = dict(default='present'),
             service_account_email = dict(),
             pem_file = dict(),
+            credentials_file = dict(),
             project_id = dict(),
         )
     )
 
+    if not HAS_LIBCLOUD:
+        module.exit_json(msg='libcloud with GCE support (0.17.0+) required for this module')
+
     gce = gce_connect(module)
 
     allowed = module.params.get('allowed')
@@ -201,7 +222,7 @@ def main():
         # user wants to create a new network that doesn't yet exist
         if name and not network:
             if not ipv4_range:
-                module.fail_json(msg="Missing required 'ipv4_range' parameter",
+                module.fail_json(msg="Network '" + name + "' is not found. To create network, 'ipv4_range' parameter is required",
                     changed=False)
 
             try:
@@ -253,7 +274,7 @@ def main():
             if fw:
                 gce.ex_destroy_firewall(fw)
                 changed = True
-        if name:
+        elif name:
             json_output['name'] = name
             network = None
             try:
@@ -272,11 +293,11 @@ def main():
                 changed = True
 
     json_output['changed'] = changed
-    print json.dumps(json_output)
-    sys.exit(0)
+    module.exit_json(**json_output)
 
 # import module snippets
 from ansible.module_utils.basic import *
 from ansible.module_utils.gce import *
 
-main()
+if __name__ == '__main__':
+    main()
