@@ -22,8 +22,6 @@ version_added: "1.4"
 short_description: Sets and retrieves file ACL information.
 description:
     - Sets and retrieves file ACL information.
-notes:
-    - As of Ansible 2.0, this module only supports Linux distributions.
 options:
   name:
     required: true
@@ -94,6 +92,7 @@ author:
     - "Jérémie Astori (@astorije)"
 notes:
     - The "acl" module requires that acls are enabled on the target filesystem and that the setfacl and getfacl binaries are installed.
+    - As of Ansible 2.0, this module only supports Linux distributions.
 '''
 
 EXAMPLES = '''
@@ -219,12 +218,9 @@ def run_acl(module, cmd, check_rc=True):
 
 
 def main():
-    if get_platform().lower() != 'linux':
-        module.fail_json(msg="The acl module is only available for Linux distributions.")
-
     module = AnsibleModule(
         argument_spec=dict(
-            name=dict(required=True, aliases=['path'], type='str'),
+            name=dict(required=True, aliases=['path'], type='path'),
             entry=dict(required=False, type='str'),
             entity=dict(required=False, type='str', default=''),
             etype=dict(
@@ -246,7 +242,10 @@ def main():
         supports_check_mode=True,
     )
 
-    path = os.path.expanduser(module.params.get('name'))
+    if get_platform().lower() != 'linux':
+        module.fail_json(msg="The acl module is only available for Linux distributions.")
+
+    path = module.params.get('name')
     entry = module.params.get('entry')
     entity = module.params.get('entity')
     etype = module.params.get('etype')
