@@ -137,9 +137,7 @@ class Hg(object):
         if not before:
             return False
 
-        args = ['update', '-C', '-R', self.dest]
-        if self.revision is not None:
-            args = args + ['-r', self.revision]
+        args = ['update', '-C', '-R', self.dest, '-r', '.']
         (rc, out, err) = self._command(args)
         if rc != 0:
             self.module.fail_json(msg=err)
@@ -213,7 +211,7 @@ def main():
     module = AnsibleModule(
         argument_spec = dict(
             repo = dict(required=True, aliases=['name']),
-            dest = dict(required=True),
+            dest = dict(required=True, type='path'),
             revision = dict(default=None, aliases=['version']),
             force = dict(default='no', type='bool'),
             purge = dict(default='no', type='bool'),
@@ -222,7 +220,7 @@ def main():
         ),
     )
     repo = module.params['repo']
-    dest = os.path.expanduser(module.params['dest'])
+    dest = module.params['dest']
     revision = module.params['revision']
     force = module.params['force']
     purge = module.params['purge']
