@@ -32,43 +32,37 @@ options:
     description:
       - "path to the mount point, eg: C(/mnt/files)"
     required: true
-    default: null
-    aliases: []
   src:
     description:
       - device to be mounted on I(name).
     required: true
-    default: null
   fstype:
     description:
       - file-system type
     required: true
-    default: null
   opts:
     description:
-      - mount options (see fstab(8))
+      - mount options (see fstab(5))
     required: false
     default: null
   dump:
     description:
-      - "dump (see fstab(8)), Note that if nulled, C(state=present) will cease to work and duplicate entries will be made with subsequent runs."
+      - "dump (see fstab(5)), Note that if nulled, C(state=present) will cease to work and duplicate entries will be made with subsequent runs."
     required: false
     default: 0
   passno:
     description:
-      - "passno (see fstab(8)), Note that if nulled, C(state=present) will cease to work and duplicate entries will be made with subsequent runs."
+      - "passno (see fstab(5)), Note that if nulled, C(state=present) will cease to work and duplicate entries will be made with subsequent runs."
     required: false
     default: 0
   state:
     description:
-      - If C(mounted) or C(unmounted), the device will be actively mounted or unmounted
-        as needed and appropriately configured in I(fstab). 
-        C(absent) and C(present) only deal with
-        I(fstab) but will not affect current mounting. If specifying C(mounted) and the mount
-        point is not present, the mount point will be created. Similarly, specifying C(absent)        will remove the mount point directory.
+      - If C(mounted) or C(unmounted), the device will be actively mounted or unmounted as needed and appropriately configured in I(fstab).
+      - C(absent) and C(present) only deal with I(fstab) but will not affect current mounting.
+      - If specifying C(mounted) and the mount point is not present, the mount point will be created. Similarly.
+      - Specifying C(absent) will remove the mount point directory.
     required: true
     choices: [ "present", "absent", "mounted", "unmounted" ]
-    default: null
   fstab:
     description:
       - file to use instead of C(/etc/fstab). You shouldn't use that option
@@ -77,9 +71,7 @@ options:
     required: false
     default: /etc/fstab
 
-notes: []
-requirements: []
-author: 
+author:
     - Ansible Core Team
     - Seth Vidal
 '''
@@ -106,7 +98,10 @@ def write_fstab(lines, dest):
 
 def _escape_fstab(v):
     """ escape space (040), ampersand (046) and backslash (134) which are invalid in fstab fields """
-    return v.replace('\\', '\\134').replace(' ', '\\040').replace('&', '\\046')
+    if isinstance(v, int):
+        return v
+    else:
+        return v.replace('\\', '\\134').replace(' ', '\\040').replace('&', '\\046')
 
 def set_mount(module, **kwargs):
     """ set/change a mount point location in fstab """
@@ -120,11 +115,6 @@ def set_mount(module, **kwargs):
     )
     args.update(kwargs)
 
-    # save the mount name before space replacement
-    origname =  args['name']
-    # replace any space in mount name with '\040' to make it fstab compatible (man fstab)
-    args['name'] = args['name'].replace(' ', r'\040')
-
     new_line = '%(src)s %(name)s %(fstype)s %(opts)s %(dump)s %(passno)s\n'
 
     to_write = []
@@ -164,14 +154,13 @@ def set_mount(module, **kwargs):
             to_write.append(line)
 
     if not exists:
-        to_write.append(new_line % args)
+        to_write.append(new_line % escaped_args)
         changed = True
 
     if changed and not module.check_mode:
         write_fstab(to_write, args['fstab'])
 
-    # mount function needs origname
-    return (origname, changed)
+    return (args['name'], changed)
 
 
 def unset_mount(module, **kwargs):
@@ -186,11 +175,6 @@ def unset_mount(module, **kwargs):
     )
     args.update(kwargs)
 
-    # save the mount name before space replacement
-    origname =  args['name']
-    # replace any space in mount name with '\040' to make it fstab compatible (man fstab)
-    args['name'] = args['name'].replace(' ', r'\040')
-
     to_write = []
     changed = False
     escaped_name = _escape_fstab(args['name'])
@@ -220,8 +204,7 @@ def unset_mount(module, **kwargs):
     if changed and not module.check_mode:
         write_fstab(to_write, args['fstab'])
 
-    # umount needs origname
-    return (origname, changed)
+    return (args['name'], changed)
 
 
 def mount(module, **kwargs):
@@ -239,10 +222,10 @@ def mount(module, **kwargs):
     mount_bin = module.get_bin_path('mount')
 
     name = kwargs['name']
-    
+
     cmd = [ mount_bin, ]
-    
-    if os.path.ismount(name):
+
+    if ismount(name):
         cmd += [ '-o', 'remount', ]
 
     if get_platform().lower() == 'freebsd':
@@ -276,7 +259,7 @@ def main():
             state  = dict(required=True, choices=['present', 'absent', 'mounted', 'unmounted']),
             name   = dict(required=True),
             opts   = dict(default=None),
-            passno = dict(default=None),
+            passno = dict(default=None, type='str'),
             dump   = dict(default=None),
             src    = dict(required=True),
             fstype = dict(required=True),
@@ -319,7 +302,7 @@ def main():
     if state == 'absent':
         name, changed = unset_mount(module, **args)
         if changed and not module.check_mode:
-            if os.path.ismount(name):
+            if ismount(name):
                 res,msg  = umount(module, **args)
                 if res:
                     module.fail_json(msg="Error unmounting %s: %s" % (name, msg))
@@ -333,7 +316,7 @@ def main():
         module.exit_json(changed=changed, **args)
 
     if state == 'unmounted':
-        if os.path.ismount(name):
+        if ismount(name):
             if not module.check_mode:
                 res,msg  = umount(module, **args)
                 if res:
@@ -353,7 +336,7 @@ def main():
         name, changed = set_mount(module, **args)
         if state == 'mounted':
             res = 0
-            if os.path.ismount(name):
+            if ismount(name):
                 if changed and not module.check_mode:
                     res,msg = mount(module, **args)
             elif 'bind' in args.get('opts', []):
@@ -380,8 +363,9 @@ def main():
         module.exit_json(changed=changed, **args)
 
     module.fail_json(msg='Unexpected position reached')
-    sys.exit(0)
 
 # import module snippets
 from ansible.module_utils.basic import *
+from ansible.module_utils.ismount import *
+
 main()
