@@ -31,6 +31,7 @@ options:
   propagating_vgw_ids:
     description:
       - "Enable route propagation from virtual gateways specified by ID."
+    default: None
     required: false
   route_table_id:
     description:
@@ -99,7 +100,7 @@ EXAMPLES = '''
       - dest: 0.0.0.0/0
         instance_id: "{{ nat.instance_id }}"
   register: nat_route_table
-  
+
 '''
 
 
@@ -253,23 +254,23 @@ def get_route_table_by_id(vpc_conn, vpc_id, route_table_id):
     route_tables = vpc_conn.get_all_route_tables(route_table_ids=[route_table_id], filters={'vpc_id': vpc_id})
     if route_tables:
         route_table = route_tables[0]
-    
+
     return route_table
-    
+
 def get_route_table_by_tags(vpc_conn, vpc_id, tags):
-    
+
     count = 0
-    route_table = None 
+    route_table = None
     route_tables = vpc_conn.get_all_route_tables(filters={'vpc_id': vpc_id})
     for table in route_tables:
         this_tags = get_resource_tags(vpc_conn, table.id)
         if tags_match(tags, this_tags):
             route_table = table
             count +=1
-    
+
     if count > 1:
         raise RuntimeError("Tags provided do not identify a unique route table")
-    else:        
+    else:
         return route_table
 
 
@@ -318,20 +319,30 @@ def ensure_routes(vpc_conn, route_table, route_specs, propagating_vgw_ids,
     # VGWs in place.
     routes_to_delete = [r for r in routes_to_match
                         if r.gateway_id != 'local'
-                        and r.gateway_id not in propagating_vgw_ids]
+                        and (propagating_vgw_ids is not None
+                             and r.gateway_id not in propagating_vgw_ids)]
 
     changed = routes_to_delete or route_specs_to_create
     if changed:
         for route_spec in route_specs_to_create:
-            vpc_conn.create_route(route_table.id,
-                                  dry_run=check_mode,
-                                  **route_spec)
+            try:
+                vpc_conn.create_route(route_table.id,
+                                      dry_run=check_mode,
+                                      **route_spec)
+            except EC2ResponseError as e:
+                if e.error_code == 'DryRunOperation':
+                    pass
 
         for route in routes_to_delete:
-            vpc_conn.delete_route(route_table.id,
-                                  route.destination_cidr_block,
-                                  dry_run=check_mode)
-    return {'changed': changed}
+            try:
+                vpc_conn.delete_route(route_table.id,
+                                      route.destination_cidr_block,
+                                      dry_run=check_mode)
+            except EC2ResponseError as e:
+                if e.error_code == 'DryRunOperation':
+                    pass
+
+    return {'changed': bool(changed)}
 
 
 def ensure_subnet_association(vpc_conn, vpc_id, route_table_id, subnet_id,
@@ -406,7 +417,6 @@ def ensure_route_table_absent(connection, module):
     route_table_id = module.params.get('route_table_id')
     tags = module.params.get('tags')
     vpc_id = module.params.get('vpc_id')
-    check_mode = module.params.get('check_mode')
 
     if lookup == 'tag':
         if tags is not None:
@@ -428,9 +438,12 @@ def ensure_route_table_absent(connection, module):
         return {'changed': False}
 
     try:
-        connection.delete_route_table(route_table.id, dry_run=check_mode)
+        connection.delete_route_table(route_table.id, dry_run=module.check_mode)
     except EC2ResponseError as e:
-        module.fail_json(msg=e.message)
+        if e.error_code == 'DryRunOperation':
+            pass
+        else:
+            module.fail_json(msg=e.message)
 
     return {'changed': True}
 
@@ -463,19 +476,18 @@ def create_route_spec(connection, routes, vpc_id):
     return routes
 
 def ensure_route_table_present(connection, module):
-    
+
     lookup = module.params.get('lookup')
-    propagating_vgw_ids = module.params.get('propagating_vgw_ids', [])
+    propagating_vgw_ids = module.params.get('propagating_vgw_ids')
     route_table_id = module.params.get('route_table_id')
     subnets = module.params.get('subnets')
     tags = module.params.get('tags')
     vpc_id = module.params.get('vpc_id')
-    check_mode = module.params.get('check_mode')
     try:
         routes = create_route_spec(connection, module.params.get('routes'), vpc_id)
     except AnsibleIgwSearchException as e:
         module.fail_json(msg=e[0])
-    
+
     changed = False
     tags_valid = False
 
@@ -494,7 +506,7 @@ def ensure_route_table_present(connection, module):
             route_table = get_route_table_by_id(connection, vpc_id, route_table_id)
         except EC2ResponseError as e:
             module.fail_json(msg=e.message)
-        
+
     # If no route table returned then create new route table
     if route_table is None:
         try:
@@ -505,10 +517,10 @@ def ensure_route_table_present(connection, module):
                 module.exit_json(changed=True)
 
             module.fail_json(msg=e.message)
-        
+
     if routes is not None:
         try:
-            result = ensure_routes(connection, route_table, routes, propagating_vgw_ids, check_mode)
+            result = ensure_routes(connection, route_table, routes, propagating_vgw_ids, module.check_mode)
             changed = changed or result['changed']
         except EC2ResponseError as e:
             module.fail_json(msg=e.message)
@@ -516,12 +528,12 @@ def ensure_route_table_present(connection, module):
     if propagating_vgw_ids is not None:
         result = ensure_propagation(connection, route_table,
                                     propagating_vgw_ids,
-                                    check_mode=check_mode)
+                                    check_mode=module.check_mode)
         changed = changed or result['changed']
 
     if not tags_valid and tags is not None:
         result = ensure_tags(connection, route_table.id, tags,
-                             add_only=True, check_mode=check_mode)
+                             add_only=True, check_mode=module.check_mode)
         changed = changed or result['changed']
 
     if subnets:
@@ -535,7 +547,7 @@ def ensure_route_table_present(connection, module):
             )
 
         try:
-            result = ensure_subnet_associations(connection, vpc_id, route_table, associated_subnets, check_mode)
+            result = ensure_subnet_associations(connection, vpc_id, route_table, associated_subnets, module.check_mode)
             changed = changed or result['changed']
         except EC2ResponseError as e:
             raise AnsibleRouteTableException(
@@ -560,18 +572,18 @@ def main():
             vpc_id = dict(default=None, required=True)
         )
     )
-    
+
     module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)
-    
+
     if not HAS_BOTO:
         module.fail_json(msg='boto is required for this module')
 
     region, ec2_url, aws_connect_params = get_aws_connection_info(module)
-    
+
     if region:
         try:
             connection = connect_to_aws(boto.vpc, region, **aws_connect_params)
-        except (boto.exception.NoAuthHandlerFound, StandardError), e:
+        except (boto.exception.NoAuthHandlerFound, AnsibleAWSError), e:
             module.fail_json(msg=str(e))
     else:
         module.fail_json(msg="region must be specified")
@@ -598,4 +610,3 @@ from ansible.module_utils.ec2 import *  # noqa
 
 if __name__ == '__main__':
     main()
-    
