@@ -89,10 +89,27 @@ options:
     default: false
   cross_zones:
     description:
-      - Whether the template should be syned across zones.
-      - Only used if C(state) is present.
+      - Whether the template should be syned or removed across zones.
+      - Only used if C(state) is present or absent.
     required: false
     default: false
+  mode:
+    description:
+      - Mode for the template extraction.
+      - Only used if C(state=extracted).
+    required: false
+    default: 'http_download'
+    choices: [ 'http_download', 'ftp_upload' ]
+  domain:
+    description:
+      - Domain the template, snapshot or VM is related to.
+    required: false
+    default: null
+  account:
+    description:
+      - Account the template, snapshot or VM is related to.
+    required: false
+    default: null
   project:
     description:
       - Name of the project the template to be registered in.
@@ -168,7 +185,7 @@ options:
   display_text:
     description:
       - Display text of the template.
-    required: true
+    required: false
     default: null
   state:
     description:
@@ -220,6 +237,7 @@ EXAMPLES = '''
 - local_action:
     module: cs_template
     name: systemvm-4.2
+    cross_zones: yes
     state: absent
 '''
 
@@ -469,6 +487,12 @@ class AnsibleCloudStackTemplate(AnsibleCloudStack):
 
 
     def register_template(self):
+        required_params = [
+            'format',
+            'url',
+            'hypervisor',
+        ]
+        self.module.fail_on_missing_params(required_params=required_params)
         template = self.get_template()
         if not template:
             self.result['changed'] = True
@@ -536,9 +560,6 @@ class AnsibleCloudStackTemplate(AnsibleCloudStack):
         args['mode']   = self.module.params.get('mode')
         args['zoneid'] = self.get_zone(key='id')
 
-        if not args['url']:
-            self.module.fail_json(msg="Missing required arguments: url")
-
         self.result['changed'] = True
 
         if not self.module.check_mode:
@@ -560,7 +581,9 @@ class AnsibleCloudStackTemplate(AnsibleCloudStack):
 
             args            = {}
             args['id']      = template['id']
-            args['zoneid']  = self.get_zone(key='id')
+
+            if not self.module.params.get('cross_zones'):
+                args['zoneid']  = self.get_zone(key='id')
 
             if not self.module.check_mode:
                 res = self.cs.deleteTemplate(**args)
@@ -592,7 +615,7 @@ def main():
         is_routing = dict(type='bool', default=False),
         checksum = dict(default=None),
         template_filter = dict(default='self', choices=['featured', 'self', 'selfexecutable', 'sharedexecutable', 'executable', 'community']),
-        hypervisor = dict(choices=['KVM', 'VMware', 'BareMetal', 'XenServer', 'LXC', 'HyperV', 'UCS', 'OVM', 'Simulator'], default=None),
+        hypervisor = dict(choices=CS_HYPERVISORS, default=None),
         requires_hvm = dict(type='bool', default=False),
         password_enabled = dict(type='bool', default=False),
         template_tag = dict(default=None),
@@ -610,16 +633,12 @@ def main():
         poll_async = dict(type='bool', default=True),
     ))
 
-    required_together = cs_required_together()
-    required_together.extend([
-        ['format', 'url', 'hypervisor'],
-    ])
-
     module = AnsibleModule(
         argument_spec=argument_spec,
-        required_together=required_together,
+        required_together=cs_required_together(),
         mutually_exclusive = (
             ['url', 'vm'],
+            ['zone', 'cross_zones'],
         ),
         supports_check_mode=True
     )
