@@ -71,6 +71,18 @@ options:
           - the port on which the consul agent is running
         required: false
         default: 8500
+    scheme:
+        description:
+          - the protocol scheme on which the consul agent is running
+        required: false
+        default: http
+        version_added: "2.1"
+    validate_certs:
+        description:
+          - whether to verify the tls certificate of the consul agent
+        required: false
+        default: True
+        version_added: "2.1"
     notes:
         description:
           - Notes to attach to check when registering it.
@@ -81,6 +93,15 @@ options:
           - the port on which the service is listening required for
             registration of a service, i.e. if service_name or service_id is set
         required: false
+    service_address:
+        description:
+          - the address to advertise that the service will be listening on.
+            This value will be passed as the I(Address) parameter to Consul's
+            U(/v1/agent/service/register) API method, so refer to the Consul API
+            documentation for further details.
+        required: false
+        default: None
+        version_added: "2.1"
     tags:
         description:
           - a list of tags that will be attached to the service registration.
@@ -166,6 +187,12 @@ EXAMPLES = '''
       interval: 60s
       http: /status
 
+  - name: register external service nginx available at 10.1.5.23
+    consul:
+      service_name: nginx
+      service_port: 80
+      service_address: 10.1.5.23
+
   - name: register nginx with some service tags
     consul:
       service_name: nginx
@@ -188,13 +215,6 @@ EXAMPLES = '''
 
 '''
 
-import sys
-
-try:
-    import json
-except ImportError:
-    import simplejson as json
-
 try:
     import consul
     from requests.exceptions import ConnectionError
@@ -280,8 +300,8 @@ def add_service(module, service):
     consul_api = get_consul_api(module)
     existing = get_service_by_id(consul_api, service.id)
 
-    # there is no way to retreive the details of checks so if a check is present
-    # in the service it must be reregistered
+    # there is no way to retrieve the details of checks so if a check is present
+    # in the service it must be re-registered
     if service.has_checks() or not existing or not existing == service:
 
         service.register(consul_api)
@@ -313,6 +333,8 @@ def remove_service(module, service_id):
 def get_consul_api(module, token=None):
     return consul.Consul(host=module.params.get('host'),
                          port=module.params.get('port'),
+                         scheme=module.params.get('scheme'),
+                         verify=module.params.get('validate_certs'),
                          token=module.params.get('token'))
 
 
@@ -351,6 +373,7 @@ def parse_service(module):
         return ConsulService(
             module.params.get('service_id'),
             module.params.get('service_name'),
+            module.params.get('service_address'),
             module.params.get('service_port'),
             module.params.get('tags'),
         )
@@ -359,13 +382,14 @@ def parse_service(module):
         module.fail_json( msg="service_name supplied but no service_port, a port is required to configure a service. Did you configure the 'port' argument meaning 'service_port'?")
 
 
-class ConsulService():
+class  ConsulService():
 
-    def __init__(self, service_id=None, name=None, port=-1,
+    def __init__(self, service_id=None, name=None, address=None, port=-1,
                  tags=None, loaded=None):
         self.id = self.name = name
         if service_id:
             self.id = service_id
+        self.address = address
         self.port = port
         self.tags = tags
         self.checks = []
@@ -382,6 +406,7 @@ class ConsulService():
             consul_api.agent.service.register(
                 self.name,
                 service_id=self.id,
+                address=self.address,
                 port=self.port,
                 tags=self.tags,
                 check=check.check)
@@ -389,6 +414,7 @@ class ConsulService():
             consul_api.agent.service.register(
                 self.name,
                 service_id=self.id,
+                address=self.address,
                 port=self.port,
                 tags=self.tags)
 
@@ -448,7 +474,7 @@ class ConsulCheck():
             self.check = consul.Check.ttl(self.ttl)
 
         if http:
-            if interval == None:
+            if interval is None:
                 raise Exception('http check must specify interval')
 
             self.check = consul.Check.http(http, self.interval, self.timeout)
@@ -493,7 +519,7 @@ class ConsulCheck():
 
     def _add(self, data, key, attr=None):
         try:
-            if attr == None:
+            if attr is None:
                 attr = key
             data[key] = getattr(self, attr)
         except:
@@ -508,6 +534,8 @@ def main():
         argument_spec=dict(
             host=dict(default='localhost'),
             port=dict(default=8500, type='int'),
+            scheme=dict(required=False, default='http'),
+            validate_certs=dict(required=False, default=True, type='bool'),
             check_id=dict(required=False),
             check_name=dict(required=False),
             check_node=dict(required=False),
@@ -516,6 +544,7 @@ def main():
             script=dict(required=False),
             service_id=dict(required=False),
             service_name=dict(required=False),
+            service_address=dict(required=False, type='str', default=None),
             service_port=dict(required=False, type='int'),
             state=dict(default='present', choices=['present', 'absent']),
             interval=dict(required=False, type='str'),
