@@ -26,7 +26,7 @@ short_description: Zabbix host creates/updates/deletes
 description:
    - This module allows you to create, modify and delete Zabbix host entries and associated group and template data.
 version_added: "2.0"
-author: 
+author:
     - "(@cove)"
     - "Tony Minfei Ding"
     - "Harrison Gu (@harrisongu)"
@@ -47,6 +47,18 @@ options:
         description:
             - Zabbix user password.
         required: true
+    http_login_user:
+        description:
+            - Basic Auth login
+        required: false
+        default: None
+        version_added: "2.1"
+    http_login_password:
+        description:
+            - Basic Auth password
+        required: false
+        default: None
+        version_added: "2.1"
     host_name:
         description:
             - Name of the host in Zabbix.
@@ -61,6 +73,13 @@ options:
             - List of templates linked to the host.
         required: false
         default: None
+    inventory_mode:
+        description:
+            - Configure the inventory mode.
+        choices: ['automatic', 'manual', 'disabled']
+        required: false
+        default: None
+        version_added: '2.1'
     status:
         description:
             - Monitoring status of the host.
@@ -116,6 +135,7 @@ EXAMPLES = '''
       - Example template2
     status: enabled
     state: present
+    inventory_mode: automatic
     interfaces:
       - type: 1
         main: 1
@@ -150,8 +170,8 @@ except ImportError:
 class ZabbixAPIExtends(ZabbixAPI):
     hostinterface = None
 
-    def __init__(self, server, timeout, **kwargs):
-        ZabbixAPI.__init__(self, server, timeout=timeout)
+    def __init__(self, server, timeout, user, passwd, **kwargs):
+        ZabbixAPI.__init__(self, server, timeout=timeout, user=user, passwd=passwd)
         self.hostinterface = ZabbixAPISubClass(self, dict({"prefix": "hostinterface"}, **kwargs))
 
 
@@ -162,13 +182,13 @@ class Host(object):
 
     # exist host
     def is_host_exist(self, host_name):
-        result = self._zapi.host.exists({'host': host_name})
+        result = self._zapi.host.get({'filter': {'host': host_name}})
         return result
 
     # check if host group exists
     def check_host_group_exist(self, group_names):
         for group_name in group_names:
-            result = self._zapi.hostgroup.exists({'name': group_name})
+            result = self._zapi.hostgroup.get({'filter': {'name': group_name}})
             if not result:
                 self._module.fail_json(msg="Hostgroup not found: %s" % group_name)
         return True
@@ -203,7 +223,9 @@ class Host(object):
         try:
             if self._module.check_mode:
                 self._module.exit_json(changed=True)
-            parameters = {'hostid': host_id, 'groups': group_ids, 'status': status, 'proxy_hostid': proxy_id}
+            parameters = {'hostid': host_id, 'groups': group_ids, 'status': status}
+            if proxy_id:
+                parameters['proxy_hostid'] = proxy_id
             self._zapi.host.update(parameters)
             interface_list_copy = exist_interface_list
             if interfaces:
@@ -238,7 +260,7 @@ class Host(object):
         try:
             if self._module.check_mode:
                 self._module.exit_json(changed=True)
-            self._zapi.host.delete({'hostid': host_id})
+            self._zapi.host.delete([host_id])
         except Exception, e:
             self._module.fail_json(msg="Failed to delete host %s: %s" % (host_name, e))
 
@@ -289,9 +311,11 @@ class Host(object):
     # check the exist_interfaces whether it equals the interfaces or not
     def check_interface_properties(self, exist_interface_list, interfaces):
         interfaces_port_list = []
-        if len(interfaces) >= 1:
-            for interface in interfaces:
-                interfaces_port_list.append(int(interface['port']))
+
+        if interfaces is not None:
+            if len(interfaces) >= 1:
+                for interface in interfaces:
+                    interfaces_port_list.append(int(interface['port']))
 
         exist_interface_ports = []
         if len(exist_interface_list) >= 1:
@@ -363,22 +387,47 @@ class Host(object):
         except Exception, e:
             self._module.fail_json(msg="Failed to link template to host: %s" % e)
 
+    # Update the host inventory_mode
+    def update_inventory_mode(self, host_id, inventory_mode):
+
+        # nothing was set, do nothing
+        if not inventory_mode:
+            return
+
+        if inventory_mode == "automatic":
+            inventory_mode = int(1)
+        elif inventory_mode == "manual":
+            inventory_mode = int(0)
+        elif inventory_mode == "disabled":
+            inventory_mode = int(-1)
+
+        # watch for - https://support.zabbix.com/browse/ZBX-6033
+        request_str = {'hostid': host_id, 'inventory_mode': inventory_mode}
+        try:
+            if self._module.check_mode:
+                self._module.exit_json(changed=True)
+            self._zapi.host.update(request_str)
+        except Exception, e:
+            self._module.fail_json(msg="Failed to set inventory_mode to host: %s" % e)
 
 def main():
     module = AnsibleModule(
         argument_spec=dict(
-            server_url=dict(required=True, aliases=['url']),
-            login_user=dict(required=True),
-            login_password=dict(required=True, no_log=True),
-            host_name=dict(required=True),
-            host_groups=dict(required=False),
-            link_templates=dict(required=False),
+            server_url=dict(type='str', required=True, aliases=['url']),
+            login_user=dict(rtype='str', equired=True),
+            login_password=dict(type='str', required=True, no_log=True),
+            host_name=dict(type='str', required=True),
+            http_login_user=dict(type='str', required=False, default=None),
+            http_login_password=dict(type='str', required=False, default=None, no_log=True),
+            host_groups=dict(type='list', required=False),
+            link_templates=dict(type='list', required=False),
             status=dict(default="enabled", choices=['enabled', 'disabled']),
             state=dict(default="present", choices=['present', 'absent']),
+            inventory_mode=dict(required=False, choices=['automatic', 'manual', 'disabled']),
             timeout=dict(type='int', default=10),
-            interfaces=dict(required=False),
-            force=dict(default=True, type='bool'),
-            proxy=dict(required=False)
+            interfaces=dict(type='list', required=False),
+            force=dict(type='bool', default=True),
+            proxy=dict(type='str', required=False)
         ),
         supports_check_mode=True
     )
@@ -389,9 +438,12 @@ def main():
     server_url = module.params['server_url']
     login_user = module.params['login_user']
     login_password = module.params['login_password']
+    http_login_user = module.params['http_login_user']
+    http_login_password = module.params['http_login_password']
     host_name = module.params['host_name']
     host_groups = module.params['host_groups']
     link_templates = module.params['link_templates']
+    inventory_mode = module.params['inventory_mode']
     status = module.params['status']
     state = module.params['state']
     timeout = module.params['timeout']
@@ -405,7 +457,7 @@ def main():
     zbx = None
     # login to zabbix
     try:
-        zbx = ZabbixAPIExtends(server_url, timeout=timeout)
+        zbx = ZabbixAPIExtends(server_url, timeout=timeout, user=http_login_user, passwd=http_login_password)
         zbx.login(login_user, login_password)
     except Exception, e:
         module.fail_json(msg="Failed to connect to Zabbix server: %s" % e)
@@ -427,15 +479,16 @@ def main():
             if interface['type'] == 1:
                 ip = interface['ip']
 
-    proxy_id = "0"
-
-    if proxy:
-        proxy_id = host.get_proxyid_by_proxy_name(proxy)
-
     # check if host exist
     is_host_exist = host.is_host_exist(host_name)
 
     if is_host_exist:
+        # Use proxy specified, or set to None when updating host
+        if proxy:
+            proxy_id = host.get_proxyid_by_proxy_name(proxy)
+        else:
+            proxy_id = None
+
         # get host id by host name
         zabbix_host_obj = host.get_host_by_host_name(host_name)
         host_id = zabbix_host_obj['hostid']
@@ -474,12 +527,23 @@ def main():
                                              exist_interfaces_copy, zabbix_host_obj, proxy_id):
                     host.update_host(host_name, group_ids, status, host_id, interfaces, exist_interfaces, proxy_id)
                     host.link_or_clear_template(host_id, template_ids)
+                    host.update_inventory_mode(host_id, inventory_mode)
                     module.exit_json(changed=True,
                                      result="Successfully update host %s (%s) and linked with template '%s'"
                                      % (host_name, ip, link_templates))
                 else:
                     module.exit_json(changed=False)
     else:
+        if state == "absent":
+            # the host is already deleted.
+            module.exit_json(changed=False)
+
+        # Use proxy specified, or set to 0 when adding new host
+        if proxy:
+            proxy_id = host.get_proxyid_by_proxy_name(proxy)
+        else:
+            proxy_id = 0
+
         if not group_ids:
             module.fail_json(msg="Specify at least one group for creating host '%s'." % host_name)
 
@@ -489,9 +553,9 @@ def main():
         # create host
         host_id = host.add_host(host_name, group_ids, status, interfaces, proxy_id)
         host.link_or_clear_template(host_id, template_ids)
+        host.update_inventory_mode(host_id, inventory_mode)
         module.exit_json(changed=True, result="Successfully added host %s (%s) and linked with template '%s'" % (
             host_name, ip, link_templates))
 
 from ansible.module_utils.basic import *
 main()
-
