@@ -20,7 +20,7 @@
 
 DOCUMENTATION = """
 ---
-author: Dag Wieers
+author: "Dag Wieers (@dagwieers)" 
 module: mail
 short_description: Send an email
 description:
@@ -62,7 +62,6 @@ options:
   subject:
     description:
       - The subject of the email being sent.
-    aliases: [ msg ]
     required: true
   body:
     description:
@@ -111,11 +110,27 @@ options:
       - The character set of email being sent
     default: 'us-ascii'
     required: false
+  subtype:
+    description:
+      - The minor mime type, can be either text or html. The major type is always text.
+    default: 'plain'
+    required: false
+    version_added: "2.0"
 """
 
 EXAMPLES = '''
 # Example playbook sending mail to root
-- local_action: mail msg='System {{ ansible_hostname }} has been successfully provisioned.'
+- local_action: mail subject='System {{ ansible_hostname }} has been successfully provisioned.'
+
+# Sending an e-mail using Gmail SMTP servers
+- local_action: mail
+                host='smtp.gmail.com'
+                port=587
+                username=username@gmail.com
+                password='mysecret'
+                to="John Smith <john.smith@example.com>"
+                subject='Ansible-report'
+                body='System {{ ansible_hostname }} has been successfully provisioned.'
 
 # Send e-mail to a bunch of users, attaching files
 - local_action: mail
@@ -129,6 +144,13 @@ EXAMPLES = '''
                 attach="/etc/group /tmp/pavatar2.png"
                 headers=Reply-To=john@example.com|X-Special="Something or other"
                 charset=utf8
+# Sending an e-mail using the remote machine, not the Ansible controller node
+- mail:
+    host='localhost'
+    port=25
+    to="John Smith <john.smith@example.com>"
+    subject='Ansible-report'
+    body='System {{ ansible_hostname }} has been successfully provisioned.'
 '''
 
 import os
@@ -156,7 +178,7 @@ def main():
     module = AnsibleModule(
         argument_spec = dict(
             username = dict(default=None),
-            password = dict(default=None),
+            password = dict(default=None, no_log=True),
             host = dict(default='localhost'),
             port = dict(default='25'),
             sender = dict(default='root', aliases=['from']),
@@ -167,7 +189,8 @@ def main():
             body = dict(default=None),
             attach = dict(default=None),
             headers = dict(default=None),
-            charset = dict(default='us-ascii')
+            charset = dict(default='us-ascii'),
+            subtype = dict(default='plain')
         )
     )
 
@@ -184,6 +207,7 @@ def main():
     attach_files = module.params.get('attach')
     headers = module.params.get('headers')
     charset = module.params.get('charset')
+    subtype = module.params.get('subtype')
     sender_phrase, sender_addr = parseaddr(sender)
 
     if not body:
@@ -243,7 +267,7 @@ def main():
     if len(cc_list) > 0:
         msg['Cc'] = ", ".join(cc_list)
 
-    part = MIMEText(body + "\n\n", _charset=charset)
+    part = MIMEText(body + "\n\n", _subtype=subtype, _charset=charset)
     msg.attach(part)
 
     if attach_files is not None:
@@ -261,7 +285,6 @@ def main():
                 msg.attach(part)
             except Exception, e:
                 module.fail_json(rc=1, msg="Failed to send mail: can't attach file %s: %s" % (file, e))
-                sys.exit()
 
     composed = msg.as_string()
 
