@@ -31,13 +31,19 @@ from ansible.compat.six import iteritems, string_types
 
 from jinja2.exceptions import UndefinedError
 
-from ansible.errors import AnsibleParserError
+from ansible.errors import AnsibleParserError, AnsibleUndefinedVariable
 from ansible.parsing.dataloader import DataLoader
 from ansible.playbook.attribute import Attribute, FieldAttribute
 from ansible.utils.boolean import boolean
 from ansible.utils.vars import combine_vars, isidentifier
 from ansible.utils.unicode import to_unicode
 
+try:
+    from __main__ import display
+except ImportError:
+    from ansible.utils.display import Display
+    display = Display()
+
 BASE_ATTRIBUTES = {}
 
 
@@ -153,7 +159,7 @@ class Base:
             setattr(Base, name, property(getter, setter, deleter))
 
             # Place the value into the instance so that the property can
-            # process and hold that value/
+            # process and hold that value.
             setattr(self, name, value.default)
 
     def preprocess_data(self, ds):
@@ -267,6 +273,8 @@ class Base:
         new_me._loader           = self._loader
         new_me._variable_manager = self._variable_manager
 
+        new_me._uuid = self._uuid
+
         # if the ds value was set on the object, copy it to the new copy too
         if hasattr(self, '_ds'):
             new_me._ds = self._ds
@@ -302,6 +310,8 @@ class Base:
                 method = getattr(self, '_post_validate_%s' % name, None)
                 if method:
                     value = method(attribute, getattr(self, name), templar)
+                elif attribute.isa == 'class':
+                    value = getattr(self, name)
                 else:
                     # if the attribute contains a variable, template it now
                     value = templar.template(getattr(self, name))
@@ -328,11 +338,18 @@ class Base:
                         if isinstance(value, string_types) and '%' in value:
                             value = value.replace('%', '')
                         value = float(value)
-                    elif attribute.isa == 'list':
+                    elif attribute.isa in ('list', 'barelist'):
                         if value is None:
                             value = []
                         elif not isinstance(value, list):
-                            value = [ value ]
+                            if isinstance(value, string_types) and attribute.isa == 'barelist':
+                                display.deprecated(
+                                    "Using comma separated values for a list has been deprecated. " \
+                                    "You should instead use the correct YAML syntax for lists. " \
+                                )
+                                value = value.split(',')
+                            else:
+                                value = [ value ]
                         if attribute.listof is not None:
                             for item in value:
                                 if not isinstance(item, attribute.listof):
@@ -344,16 +361,24 @@ class Base:
                     elif attribute.isa == 'set':
                         if value is None:
                             value = set()
-                        else:
-                            if not isinstance(value, (list, set)):
+                        elif not isinstance(value, (list, set)):
+                            if isinstance(value, string_types):
+                                value = value.split(',')
+                            else:
+                                # Making a list like this handles strings of
+                                # text and bytes properly
                                 value = [ value ]
-                            if not isinstance(value, set):
-                                value = set(value)
+                        if not isinstance(value, set):
+                            value = set(value)
                     elif attribute.isa == 'dict':
                         if value is None:
                             value = dict()
                         elif not isinstance(value, dict):
                             raise TypeError("%s is not a dictionary" % value)
+                    elif attribute.isa == 'class':
+                        if not isinstance(value, attribute.class_type):
+                            raise TypeError("%s is not a valid %s (got a %s instead)" % (name, attribute.class_type, type(value)))
+                        value.post_validate(templar=templar)
 
                 # and assign the massaged value back to the attribute field
                 setattr(self, name, value)
@@ -361,7 +386,7 @@ class Base:
             except (TypeError, ValueError) as e:
                 raise AnsibleParserError("the field '%s' has an invalid value (%s), and could not be converted to an %s."
                         " Error was: %s" % (name, value, attribute.isa, e), obj=self.get_ds())
-            except UndefinedError as e:
+            except (AnsibleUndefinedVariable, UndefinedError) as e:
                 if templar._fail_on_undefined_errors and name != 'name':
                     raise AnsibleParserError("the field '%s' has an invalid value, which appears to include a variable that is undefined."
                             " The error was: %s" % (name,e), obj=self.get_ds())
@@ -414,7 +439,7 @@ class Base:
         def _validate_variable_keys(ds):
             for key in ds:
                 if not isidentifier(key):
-                    raise TypeError("%s is not a valid variable name" % key)
+                    raise TypeError("'%s' is not a valid variable name" % key)
 
         try:
             if isinstance(ds, dict):
