@@ -19,15 +19,16 @@
 from __future__ import (absolute_import, division, print_function)
 __metaclass__ = type
 
-from ansible.compat.six.moves import queue as Queue
-from ansible.compat.six import iteritems, text_type, string_types
-
 import json
 import time
 import zlib
+from collections import defaultdict
+from multiprocessing import Lock
 
 from jinja2.exceptions import UndefinedError
 
+from ansible.compat.six.moves import queue as Queue
+from ansible.compat.six import iteritems, text_type, string_types
 from ansible import constants as C
 from ansible.errors import AnsibleError, AnsibleParserError, AnsibleUndefinedVariable
 from ansible.executor.play_iterator import PlayIterator
@@ -35,11 +36,14 @@ from ansible.executor.process.worker import WorkerProcess
 from ansible.executor.task_result import TaskResult
 from ansible.inventory.host import Host
 from ansible.inventory.group import Group
+from ansible.module_utils.facts import Facts
 from ansible.playbook.helpers import load_list_of_blocks
 from ansible.playbook.included_file import IncludedFile
 from ansible.plugins import action_loader, connection_loader, filter_loader, lookup_loader, module_loader, test_loader
 from ansible.template import Templar
+from ansible.utils.unicode import to_unicode
 from ansible.vars.unsafe_proxy import wrap_var
+from ansible.vars import combine_vars
 
 try:
     from __main__ import display
@@ -49,6 +53,24 @@ except ImportError:
 
 __all__ = ['StrategyBase']
 
+if 'action_write_locks' not in globals():
+    # Do not initialize this more than once because it seems to bash
+    # the existing one.  multiprocessing must be reloading the module
+    # when it forks?
+    action_write_locks = dict()
+
+    # Below is a Lock for use when we weren't expecting a named module.
+    # It gets used when an action plugin directly invokes a module instead
+    # of going through the strategies.  Slightly less efficient as all
+    # processes with unexpected module names will wait on this lock
+    action_write_locks[None] = Lock()
+
+    # These plugins are called directly by action plugins (not going through
+    # a strategy).  We precreate them here as an optimization
+    mods = set(p['name'] for p in Facts.PKG_MGRS)
+    mods.update(('copy', 'file', 'setup', 'slurp', 'stat'))
+    for mod_name in mods:
+        action_write_locks[mod_name] = Lock()
 
 # TODO: this should probably be in the plugins/__init__.py, with
 #       a smarter mechanism to set all of the attributes based on
@@ -98,7 +120,7 @@ class StrategyBase:
     def run(self, iterator, play_context, result=True):
         # save the failed/unreachable hosts, as the run_handlers()
         # method will clear that information during its execution
-        failed_hosts      = self._tqm._failed_hosts.keys()
+        failed_hosts      = iterator.get_failed_hosts()
         unreachable_hosts = self._tqm._unreachable_hosts.keys()
 
         display.debug("running handlers")
@@ -106,18 +128,20 @@ class StrategyBase:
 
         # now update with the hosts (if any) that failed or were
         # unreachable during the handler execution phase
-        failed_hosts      = set(failed_hosts).union(self._tqm._failed_hosts.keys())
+        failed_hosts      = set(failed_hosts).union(iterator.get_failed_hosts())
         unreachable_hosts = set(unreachable_hosts).union(self._tqm._unreachable_hosts.keys())
 
         # return the appropriate code, depending on the status hosts after the run
-        if len(unreachable_hosts) > 0:
-            return 3
+        if not isinstance(result, bool) and result != self._tqm.RUN_OK:
+            return result
+        elif len(unreachable_hosts) > 0:
+            return self._tqm.RUN_UNREACHABLE_HOSTS
         elif len(failed_hosts) > 0:
-            return 2
-        elif not result:
-            return 1
+            return self._tqm.RUN_FAILED_HOSTS
+        elif isinstance(result, bool) and not result:
+            return self._tqm.RUN_ERROR
         else:
-            return 0
+            return self._tqm.RUN_OK
 
     def get_hosts_remaining(self, play):
         return [host for host in self._inventory.get_hosts(play.hosts)
@@ -139,7 +163,21 @@ class StrategyBase:
 
         display.debug("entering _queue_task() for %s/%s" % (host, task))
 
-        task_vars['hostvars'] = self._tqm.hostvars
+        # Add a write lock for tasks.
+        # Maybe this should be added somewhere further up the call stack but
+        # this is the earliest in the code where we have task (1) extracted
+        # into its own variable and (2) there's only a single code path
+        # leading to the module being run.  This is called by three
+        # functions: __init__.py::_do_handler_run(), linear.py::run(), and
+        # free.py::run() so we'd have to add to all three to do it there.
+        # The next common higher level is __init__.py::run() and that has
+        # tasks inside of play_iterator so we'd have to extract them to do it
+        # there.
+        global action_write_locks
+        if task.action not in action_write_locks:
+            display.debug('Creating lock for %s' % task.action)
+            action_write_locks[task.action] = Lock()
+
         # and then queue the new task
         display.debug("%s - putting task (%s) in queue" % (host, task))
         try:
@@ -151,7 +189,7 @@ class StrategyBase:
 
             queued = False
             while True:
-                (worker_prc, main_q, rslt_q) = self._workers[self._cur_worker]
+                (worker_prc, rslt_q) = self._workers[self._cur_worker]
                 if worker_prc is None or not worker_prc.is_alive():
                     worker_prc = WorkerProcess(rslt_q, task_vars, host, task, play_context, self._loader, self._variable_manager, shared_loader_obj)
                     self._workers[self._cur_worker][0] = worker_prc
@@ -208,10 +246,26 @@ class StrategyBase:
                                 [iterator.mark_host_failed(h) for h in self._inventory.get_hosts(iterator._play.hosts) if h.name not in self._tqm._unreachable_hosts]
                             else:
                                 iterator.mark_host_failed(host)
-                            (state, tmp_task) = iterator.get_next_task_for_host(host, peek=True)
-                            if not state or state.run_state != PlayIterator.ITERATING_RESCUE:
+
+                            # only add the host to the failed list officially if it has
+                            # been failed by the iterator
+                            if iterator.is_failed(host):
                                 self._tqm._failed_hosts[host.name] = True
                                 self._tqm._stats.increment('failures', host.name)
+                            else:
+                                # otherwise, we grab the current state and if we're iterating on
+                                # the rescue portion of a block then we save the failed task in a
+                                # special var for use within the rescue/always
+                                state, _ = iterator.get_next_task_for_host(host, peek=True)
+                                if state.run_state == iterator.ITERATING_RESCUE:
+                                    original_task = iterator.get_original_task(host, task)
+                                    self._variable_manager.set_nonpersistent_facts(
+                                        host,
+                                        dict(
+                                            ansible_failed_task=original_task.serialize(),
+                                            ansible_failed_result=task_result._result,
+                                        ),
+                                    )
                         else:
                             self._tqm._stats.increment('ok', host.name)
                         self._tqm.send_callback('v2_runner_on_failed', task_result, ignore_errors=task.ignore_errors)
@@ -264,12 +318,47 @@ class StrategyBase:
 
                     original_host = get_original_host(task_result._host)
                     original_task = iterator.get_original_task(original_host, task_result._task)
-                    if handler_name not in self._notified_handlers:
-                        self._notified_handlers[handler_name] = []
 
-                    if original_host not in self._notified_handlers[handler_name]:
-                        self._notified_handlers[handler_name].append(original_host)
-                        display.vv("NOTIFIED HANDLER %s" % (handler_name,))
+                    def search_handler_blocks(handler_blocks):
+                        for handler_block in handler_blocks:
+                            for handler_task in handler_block.block:
+                                handler_vars = self._variable_manager.get_vars(loader=self._loader, play=iterator._play, task=handler_task)
+                                templar = Templar(loader=self._loader, variables=handler_vars)
+                                try:
+                                    # first we check with the full result of get_name(), which may
+                                    # include the role name (if the handler is from a role). If that
+                                    # is not found, we resort to the simple name field, which doesn't
+                                    # have anything extra added to it.
+                                    target_handler_name = templar.template(handler_task.name)
+                                    if target_handler_name == handler_name:
+                                        return handler_task
+                                    else:
+                                        target_handler_name = templar.template(handler_task.get_name())
+                                        if target_handler_name == handler_name:
+                                            return handler_task
+                                except (UndefinedError, AnsibleUndefinedVariable):
+                                    # We skip this handler due to the fact that it may be using
+                                    # a variable in the name that was conditionally included via
+                                    # set_fact or some other method, and we don't want to error
+                                    # out unnecessarily
+                                    continue
+                        return None
+
+                    # Find the handler using the above helper.  First we look up the
+                    # dependency chain of the current task (if it's from a role), otherwise
+                    # we just look through the list of handlers in the current play/all
+                    # roles and use the first one that matches the notify name
+                    target_handler = search_handler_blocks(iterator._play.handlers)
+                    if target_handler is None:
+                        raise AnsibleError("The requested handler '%s' was not found in any of the known handlers" % handler_name)
+
+                    if target_handler in self._notified_handlers:
+                        if original_host not in self._notified_handlers[target_handler]:
+                            self._notified_handlers[target_handler].append(original_host)
+                            # FIXME: should this be a callback?
+                            display.vv("NOTIFIED HANDLER %s" % (handler_name,))
+                    else:
+                        raise AnsibleError("The requested handler '%s' was not found in the main handlers list" % handler_name)
 
                 elif result[0] == 'register_host_var':
                     # essentially the same as 'set_host_var' below, however we
@@ -298,8 +387,11 @@ class StrategyBase:
                     if task.delegate_to is not None and task.delegate_facts:
                         task_vars = self._variable_manager.get_vars(loader=self._loader, play=iterator._play, host=host, task=task)
                         self.add_tqm_variables(task_vars, play=iterator._play)
+                        loop_var = 'item'
+                        if task.loop_control:
+                            loop_var = task.loop_control.loop_var or 'item'
                         if item is not None:
-                            task_vars['item'] = item
+                            task_vars[loop_var] = item
                         templar = Templar(loader=self._loader, variables=task_vars)
                         host_name = templar.template(task.delegate_to)
                         actual_host = self._inventory.get_host(host_name)
@@ -308,24 +400,28 @@ class StrategyBase:
                     else:
                         actual_host = host
 
+                    if task.run_once:
+                        host_list = [host for host in self._inventory.get_hosts(iterator._play.hosts) if host.name not in self._tqm._unreachable_hosts]
+                    else:
+                        host_list = [actual_host]
+
                     if result[0] == 'set_host_var':
                         var_name  = result[4]
                         var_value = result[5]
-
-                        if task.run_once:
-                            host_list = [host for host in self._inventory.get_hosts(iterator._play.hosts) if host.name not in self._tqm._unreachable_hosts]
-                        else:
-                            host_list = [actual_host]
-
                         for target_host in host_list:
                             self._variable_manager.set_host_variable(target_host, var_name, var_value)
                     elif result[0] == 'set_host_facts':
                         facts = result[4]
-                        if task.action == 'set_fact':
-                            self._variable_manager.set_nonpersistent_facts(actual_host, facts)
-                        else:
-                            self._variable_manager.set_host_facts(actual_host, facts)
-
+                        for target_host in host_list:
+                            if task.action == 'set_fact':
+                                self._variable_manager.set_nonpersistent_facts(target_host, facts.copy())
+                            else:
+                                self._variable_manager.set_host_facts(target_host, facts.copy())
+                elif result[0].startswith('v2_runner_item') or result[0] == 'v2_runner_retry':
+                    self._tqm.send_callback(result[0], result[1])
+                elif result[0] == 'v2_on_file_diff':
+                    if self._diff:
+                        self._tqm.send_callback('v2_on_file_diff', result[1])
                 else:
                     raise AnsibleError("unknown result message received: %s" % result[0])
 
@@ -347,9 +443,14 @@ class StrategyBase:
 
         display.debug("waiting for pending results...")
         while self._pending_results > 0 and not self._tqm._terminated:
+
+            if self._tqm.has_dead_workers():
+                raise AnsibleError("A worker was found in a dead state")
+
             results = self._process_pending_results(iterator)
             ret_results.extend(results)
-            time.sleep(0.0001)
+            time.sleep(0.005)
+
         display.debug("no more pending results, returning what we have")
 
         return ret_results
@@ -361,26 +462,26 @@ class StrategyBase:
 
         host_name = host_info.get('host_name')
 
-        # Check if host in cache, add if not
-        if host_name in self._inventory._hosts_cache:
-            new_host = self._inventory._hosts_cache[host_name]
-        else:
+        # Check if host in inventory, add if not
+        new_host = self._inventory.get_host(host_name)
+        if not new_host:
             new_host = Host(name=host_name)
             self._inventory._hosts_cache[host_name] = new_host
+            self._inventory.get_host_vars(new_host)
 
             allgroup = self._inventory.get_group('all')
             allgroup.add_host(new_host)
 
         # Set/update the vars for this host
-        new_vars = host_info.get('host_vars', dict())
-        new_host.vars = self._inventory.get_host_vars(new_host)
-        new_host.vars.update(new_vars)
+        new_host.vars = combine_vars(new_host.vars, self._inventory.get_host_vars(new_host))
+        new_host.vars = combine_vars(new_host.vars,  host_info.get('host_vars', dict()))
 
         new_groups = host_info.get('groups', [])
         for group_name in new_groups:
             if not self._inventory.get_group(group_name):
                 new_group = Group(group_name)
                 self._inventory.add_group(new_group)
+                self._inventory.get_group_vars(new_group)
                 new_group.vars = self._inventory.get_group_variables(group_name)
             else:
                 new_group = self._inventory.get_group(group_name)
@@ -449,11 +550,12 @@ class StrategyBase:
             block_list = load_list_of_blocks(
                 data,
                 play=included_file._task._block._play,
-                parent_block=included_file._task._block,
+                parent_block=None,
                 task_include=included_file._task,
                 role=included_file._task._role,
                 use_handlers=is_handler,
-                loader=self._loader
+                loader=self._loader,
+                variable_manager=self._variable_manager,
             )
 
             # since we skip incrementing the stats when the task result is
@@ -465,7 +567,7 @@ class StrategyBase:
             # mark all of the hosts including this file as failed, send callbacks,
             # and increment the stats for this host
             for host in included_file._hosts:
-                tr = TaskResult(host=host, task=included_file._task, return_data=dict(failed=True, reason=str(e)))
+                tr = TaskResult(host=host, task=included_file._task, return_data=dict(failed=True, reason=to_unicode(e)))
                 iterator.mark_host_failed(host)
                 self._tqm._failed_hosts[host.name] = True
                 self._tqm._stats.increment('failures', host.name)
@@ -475,11 +577,7 @@ class StrategyBase:
         # set the vars for this task from those specified as params to the include
         for b in block_list:
             # first make a copy of the including task, so that each has a unique copy to modify
-            # FIXME: not sure if this is the best way to fix this, as we might be losing
-            #        information in the copy. Previously we assigned the include params to
-            #        the block variables directly, which caused other problems, so we may
-            #        need to figure out a third option if this also presents problems.
-            b._task_include = b._task_include.copy(exclude_block=True)
+            b._task_include = b._task_include.copy()
             # then we create a temporary set of vars to ensure the variable reference is unique
             temp_vars = b._task_include.vars.copy()
             temp_vars.update(included_file._args.copy())
@@ -488,10 +586,10 @@ class StrategyBase:
             # error so that users know not to specify them both ways
             tags = temp_vars.pop('tags', [])
             if isinstance(tags, string_types):
-                tags = [ tags ]
+                tags = tags.split(',')
             if len(tags) > 0:
                 if len(b._task_include.tags) > 0:
-                    raise AnsibleParserError("Include tasks should not specify tags in more than one way (both via args and directly on the task)",
+                    raise AnsibleParserError("Include tasks should not specify tags in more than one way (both via args and directly on the task). Mixing tag specify styles is prohibited for whole import hierarchy, not only for single import statement",
                             obj=included_file._task._ds)
                 display.deprecated("You should not specify tags in the include parameters. All tags should be specified using the task-level option")
                 b._task_include.tags = tags
@@ -514,25 +612,8 @@ class StrategyBase:
             #        but this may take some work in the iterator and gets tricky when
             #        we consider the ability of meta tasks to flush handlers
             for handler in handler_block.block:
-                handler_vars = self._variable_manager.get_vars(loader=self._loader, play=iterator._play, task=handler)
-                templar = Templar(loader=self._loader, variables=handler_vars)
-                try:
-                    # first we check with the full result of get_name(), which may
-                    # include the role name (if the handler is from a role). If that
-                    # is not found, we resort to the simple name field, which doesn't
-                    # have anything extra added to it.
-                    handler_name = templar.template(handler.name)
-                    if handler_name not in self._notified_handlers:
-                        handler_name = templar.template(handler.get_name())
-                except (UndefinedError, AnsibleUndefinedVariable):
-                    # We skip this handler due to the fact that it may be using
-                    # a variable in the name that was conditionally included via
-                    # set_fact or some other method, and we don't want to error
-                    # out unnecessarily
-                    continue
-
-                if handler_name in self._notified_handlers and len(self._notified_handlers[handler_name]):
-                    result = self._do_handler_run(handler, handler_name, iterator=iterator, play_context=play_context)
+                if handler in self._notified_handlers and len(self._notified_handlers[handler]):
+                    result = self._do_handler_run(handler, handler.get_name(), iterator=iterator, play_context=play_context)
                     if not result:
                         break
         return result
@@ -544,10 +625,13 @@ class StrategyBase:
         #    self._tqm.send_callback('v2_playbook_on_no_hosts_remaining')
         #    result = False
         #    break
+        saved_name = handler.name
+        handler.name = handler_name
         self._tqm.send_callback('v2_playbook_on_handler_task_start', handler)
+        handler.name = saved_name
 
         if notified_hosts is None:
-            notified_hosts = self._notified_handlers[handler_name]
+            notified_hosts = self._notified_handlers[handler]
 
         run_once = False
         try:
@@ -610,17 +694,17 @@ class StrategyBase:
                     continue
 
         # wipe the notification list
-        self._notified_handlers[handler_name] = []
+        self._notified_handlers[handler] = []
         display.debug("done running handlers, result is: %s" % result)
         return result
 
     def _take_step(self, task, host=None):
 
         ret=False
+        msg=u'Perform task: %s ' % task
         if host:
-            msg = u'Perform task: %s on %s (y/n/c): ' % (task, host)
-        else:
-            msg = u'Perform task: %s (y/n/c): ' % task
+            msg += u'on %s ' % host
+        msg += u'(N)o/(y)es/(c)ontinue: '
         resp = display.prompt(msg)
 
         if resp.lower() in ['y','yes']:
@@ -650,7 +734,15 @@ class StrategyBase:
             self.run_handlers(iterator, play_context)
         elif meta_action == 'refresh_inventory':
             self._inventory.refresh_inventory()
+        elif meta_action == 'clear_facts':
+            for host in iterator._host_states:
+                self._variable_manager.clear_facts(host)
         #elif meta_action == 'reset_connection':
         #    connection_info.connection.close()
+        elif meta_action == 'clear_host_errors':
+            self._tqm._failed_hosts = dict()
+            self._tqm._unreachable_hosts = dict()
+            for host in iterator._host_states:
+                iterator._host_states[host].fail_state = iterator.FAILED_NONE
         else:
             raise AnsibleError("invalid meta action requested: %s" % meta_action, obj=task._ds)
